module Functions.Date
  ( Date
  , extractDate
  , formatDate
  , formatDateShort
  ) where

import Types (Token)
import DependencyGenerator (DepGenM, TokenableTo(..), onToken)

import Text.Printf (printf)
import qualified Data.Text as T
import Data.List.NonEmpty (NonEmpty(..))
import qualified Data.List.NonEmpty as NE
import Data.Text (Text)

data Date = Date Int Int Int
  deriving (Eq, Ord, Show)

split :: Eq a => a -> [a] -> NonEmpty [a]
split sep = \case
  [] ->
    NE.singleton []
  (c : cs) ->
    (if sep == c
     then NE.cons []
     else \(h :| t) -> (c : h) :| t)
    $ split sep cs

extractDate :: TokenableTo String a => a -> DepGenM (Token Date)
extractDate = onToken $ \dirName -> case split '-' dirName of
  year :| (month : day : _) ->
    Date (read year) (read month) (read day)
  _ ->
    error "unexpected"

formatDate :: Date -> Text
formatDate (Date year month day) =
  T.concat [ "den "
           , T.pack (show day)
           , ". "
           , months !! (month - 1)
           , " "
           , T.pack (show year)
           ]
  where months = [ "januar"
                 , "februar"
                 , "marts"
                 , "april"
                 , "maj"
                 , "juni"
                 , "juli"
                 , "august"
                 , "september"
                 , "oktober"
                 , "november"
                 , "december"
                 ]

formatDateShort :: Date -> Text
formatDateShort (Date year month day) =
  T.concat [ T.pack (show year), "-", T.pack (printf "%02d" month), "-", T.pack (printf "%02d" day) ]
